package com.loafle.totopia.member.controller;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.validation.Valid;

import com.loafle.totopia.commons.exception.ResourceNotFoundException;
import com.loafle.totopia.member.dao.MemberRepository;
import com.loafle.totopia.member.model.Member;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping("/api/v1")
public class MemberController {
  @Autowired
  private MemberRepository memberRepository;

  @GetMapping("/members")
  public List<Member> getAllMembers() {
    return memberRepository.findAll();
  }

  @GetMapping("/members/{id}")
  public ResponseEntity<Member> getMemberById(@PathVariable(value = "id") Long memberId)
      throws ResourceNotFoundException {
    Member member = memberRepository.findById(memberId)
        .orElseThrow(() -> new ResourceNotFoundException("Member not found for this id :: " + memberId));
    return ResponseEntity.ok().body(member);
  }

  @PostMapping(path = "/members", produces = MediaType.APPLICATION_JSON_VALUE)
  public Member createMember(@Valid @RequestBody Member member) {
    return memberRepository.save(member);
  }

  @PutMapping("/members/{id}")
  public ResponseEntity<Member> updateMember(@PathVariable(value = "id") Long memberId,
      @Valid @RequestBody Member memberDetails) throws ResourceNotFoundException {
    Member member = memberRepository.findById(memberId)
        .orElseThrow(() -> new ResourceNotFoundException("Member not found for this id :: " + memberId));

    member.setEmailId(memberDetails.getEmailId());

    final Member updatedMember = memberRepository.save(member);
    return ResponseEntity.ok(updatedMember);
  }

  @DeleteMapping("/members/{id}")
  public Map<String, Boolean> deleteMember(@PathVariable(value = "id") Long memberId) throws ResourceNotFoundException {
    Member member = memberRepository.findById(memberId)
        .orElseThrow(() -> new ResourceNotFoundException("Member not found for this id :: " + memberId));

    memberRepository.delete(member);
    Map<String, Boolean> response = new HashMap<>();
    response.put("deleted", Boolean.TRUE);
    return response;
  }
}